@@ -168,5 +168,9 @@ pub fn configure_window(winit_window: &Window, config: &Config) {
         winit_window.set_has_shadow(!is_transparent);
     }
 
+    if let Some(title) = &config.window.initial_title {
+        winit_window.set_title(title);
+    }
+
     winit_window.set_blur(config.window.blur);
 }
